@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2010, 2011 Marco Peereboom <marco@peereboom.us>
  * Copyright (c) 2011 Stevan Andjelkovic <stevan@student.chalmers.se>
- * Copyright (c) 2010, 2011 Edd Barrett <vext01@gmail.com>
+ * Copyright (c) 2010, 2011, 2012 Edd Barrett <vext01@gmail.com>
  * Copyright (c) 2011 Todd T. Fries <todd@fries.net>
  * Copyright (c) 2011 Raphael Graf <r@undefined.ch>
  * Copyright (c) 2011 Michal Mazurek <akfaew@jasminek.net>
@@ -19,7 +19,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include "xxxterm.h"
+#include <xxxterm.h>
 
 /*
  * xxxterm "protocol" (xtp)
@@ -71,6 +71,7 @@
 #define XT_XTP_DL_CANCEL	(2)
 #define XT_XTP_DL_REMOVE	(3)
 #define XT_XTP_DL_UNLINK	(4)
+#define XT_XTP_DL_START		(5)
 
 /* XTP history actions */
 #define XT_XTP_HL_LIST		(1)
@@ -80,6 +81,7 @@
 #define XT_XTP_CL_LIST		(1)
 #define XT_XTP_CL_REMOVE	(2)
 #define XT_XTP_CL_REMOVE_DOMAIN	(3)
+#define XT_XTP_CL_REMOVE_ALL	(4)
 
 /* XTP cookie actions */
 #define XT_XTP_FL_LIST		(1)
@@ -91,9 +93,13 @@ int			set(struct tab *, struct karg *);
 int			marco(struct tab *, struct karg *);
 int			startpage(struct tab *, struct karg *);
 const char *		marco_message(int *);
+void			update_cookie_tabs(struct tab *apart_from);
+int			about_webkit(struct tab *, struct karg *);
+int			allthethings(struct tab *, struct karg *);
 
 struct about_type about_list[] = {
 	{ XT_URI_ABOUT_ABOUT,		about },
+	{ XT_URI_ABOUT_ALLTHETHINGS,	allthethings },
 	{ XT_URI_ABOUT_BLANK,		blank },
 	{ XT_URI_ABOUT_CERTS,		ca_cmd },
 	{ XT_URI_ABOUT_COOKIEWL,	cookie_show_wl },
@@ -108,6 +114,7 @@ struct about_type about_list[] = {
 	{ XT_URI_ABOUT_MARCO,		marco },
 	{ XT_URI_ABOUT_STARTPAGE,	startpage },
 	{ XT_URI_ABOUT_PLUGINWL,	pl_show_wl },
+	{ XT_URI_ABOUT_WEBKIT,		about_webkit },
 };
 
 /*
@@ -117,7 +124,8 @@ struct about_type about_list[] = {
  */
 #define XT_XTP_SES_KEY_SZ	8
 #define XT_XTP_SES_KEY_HEX_FMT  \
-	"%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx"
+	"%02" PRIx8 "%02" PRIx8 "%02" PRIx8 "%02" PRIx8 "%02" PRIx8 "%02" PRIx8 "%02" PRIx8 "%02" PRIx8
+
 char			*dl_session_key;	/* downloads */
 char			*hl_session_key;	/* history list */
 char			*cl_session_key;	/* cookie list */
@@ -187,7 +195,7 @@ load_webkit_string(struct tab *t, const char *str, gchar *title)
 		gtk_spinner_stop(GTK_SPINNER(t->spinner));
 		gtk_widget_hide(t->spinner);
 #endif
-		snprintf(file, sizeof file, "%s/%s", resource_dir, icons[0]);
+		snprintf(file, sizeof file, "%s" PS "%s", resource_dir, icons[0]);
 		xt_icon_from_file(t, file);
 	}
 }
@@ -214,6 +222,11 @@ about(struct tab *t, struct karg *args)
 	body = g_strdup_printf("<b>Version: %s</b>"
 #ifdef XXXTERM_BUILDSTR
 	    "<br><b>Build: %s</b>"
+#endif
+	    "<br><b>WebKit: %d.%d.%d</b>"
+	    "<br><b>User Agent: %d.%d</b>"
+#ifdef WEBKITGTK_API_VERSION
+	    "<br><b>WebKit API: %.1f</b>"
 #endif
 	    "<p>"
 	    "Authors:"
@@ -229,9 +242,14 @@ about(struct tab *t, struct karg *args)
 	    "<a href=\"http://opensource.conformal.com/wiki/XXXTerm\">website</a>"
 	    "</p>",
 #ifdef XXXTERM_BUILDSTR
-	    version, XXXTERM_BUILDSTR
+	    version, XXXTERM_BUILDSTR,
 #else
-	    version
+	    version,
+#endif
+	    WEBKIT_MAJOR_VERSION, WEBKIT_MINOR_VERSION, WEBKIT_MICRO_VERSION,
+	    WEBKIT_USER_AGENT_MAJOR_VERSION, WEBKIT_USER_AGENT_MINOR_VERSION
+#ifdef WEBKITGTK_API_VERSION
+	    ,WEBKITGTK_API_VERSION
 #endif
 	    );
 
@@ -271,7 +289,7 @@ int
 stats(struct tab *t, struct karg *args)
 {
 	char			*page, *body, *s, line[64 * 1024];
-	long long unsigned int	line_count = 0;
+	uint64_t		line_count = 0;
 	FILE			*r_cookie_f;
 
 	if (t == NULL)
@@ -289,14 +307,15 @@ stats(struct tab *t, struct karg *args)
 			}
 			fclose(r_cookie_f);
 			snprintf(line, sizeof line,
-			    "<br/>Cookies blocked(*) total: %llu", line_count);
+			    "<br/>Cookies blocked(*) total: %" PRIu64,
+			    line_count);
 		} else
 			show_oops(t, "Can't open blocked cookies file: %s",
 			    strerror(errno));
 	}
 
 	body = g_strdup_printf(
-	    "Cookies blocked(*) this session: %llu"
+	    "Cookies blocked(*) this session: %" PRIu64
 	    "%s"
 	    "<p><small><b>*</b> results vary based on settings</small></p>",
 	    blocked_cookies,
@@ -424,8 +443,10 @@ cookie_cmd(struct tab *t, struct karg *args)
 	} else if (args->i & XT_SAVE) {
 		args->i |= XT_WL_RELOAD;
 		wl_save(t, args, XT_WL_COOKIE);
-	} else if (args->i & XT_DELETE)
-		show_oops(t, "'cookie delete' currently unimplemented");
+	} else if (args->i & XT_DELETE) {
+		remove_cookie_all();
+		update_cookie_tabs(NULL);
+	}
 
 	return (0);
 }
@@ -497,13 +518,24 @@ xtp_handle_dl(struct tab *t, uint8_t cmd, int id)
 
 	/* decide what to do */
 	switch (cmd) {
+	case XT_XTP_DL_START:
+		/* our downloads always needs to be
+		 * restarted if called from here
+		 */
+		download_start(t, d, XT_DL_RESTART);
+		break;
 	case XT_XTP_DL_CANCEL:
 		webkit_download_cancel(d->download);
 		g_object_unref(d->download);
+		RB_REMOVE(download_list, &downloads, d);
 		break;
 	case XT_XTP_DL_UNLINK:
+#ifdef __MINGW32__
+		unlink(webkit_download_get_destination_uri(d->download));
+#else
 		unlink(webkit_download_get_destination_uri(d->download) +
 		    strlen("file://"));
+#endif
 		/* FALLTHROUGH */
 	case XT_XTP_DL_REMOVE:
 		webkit_download_cancel(d->download); /* just incase */
@@ -567,7 +599,7 @@ remove_favorite(struct tab *t, int index)
 	size_t			len, lineno;
 
 	/* open favorites */
-	snprintf(file, sizeof file, "%s/%s", work_dir, XT_FAVS_FILE);
+	snprintf(file, sizeof file, "%s" PS "%s", work_dir, XT_FAVS_FILE);
 
 	if ((f = fopen(file, "r")) == NULL) {
 		show_oops(t, "%s: can't open favorites: %s",
@@ -651,7 +683,7 @@ add_favorite(struct tab *t, struct karg *args)
 		return (1);
 	}
 
-	snprintf(file, sizeof file, "%s/%s", work_dir, XT_FAVS_FILE);
+	snprintf(file, sizeof file, "%s" PS "%s", work_dir, XT_FAVS_FILE);
 	if ((f = fopen(file, "r+")) == NULL) {
 		show_oops(t, "Can't open favorites file: %s", strerror(errno));
 		return (1);
@@ -721,6 +753,9 @@ xtp_handle_cl(struct tab *t, uint8_t cmd, int arg)
 	case XT_XTP_CL_REMOVE_DOMAIN:
 		remove_cookie_domain(arg);
 		break;
+	case XT_XTP_CL_REMOVE_ALL:
+		remove_cookie_all();
+		break;
 	default:
 		show_oops(t, "%s: unknown cookie xtp command", __func__);
 		break;
@@ -957,7 +992,7 @@ xtp_page_fl(struct tab *t, struct karg *args)
 		generate_xtp_session_key(&fl_session_key);
 
 	/* open favorites */
-	snprintf(file, sizeof file, "%s/%s", work_dir, XT_FAVS_FILE);
+	snprintf(file, sizeof file, "%s" PS "%s", work_dir, XT_FAVS_FILE);
 	if ((f = fopen(file, "r")) == NULL) {
 		show_oops(t, "Can't open favorites file: %s", strerror(errno));
 		return (1);
@@ -1047,6 +1082,7 @@ xtp_page_dl_row(struct tab *t, char *html, struct download *dl)
 {
 
 	WebKitDownloadStatus	stat;
+	const gchar		*destination;
 	char			*status_html = NULL, *cmd_html = NULL, *new_html;
 	gdouble			progress;
 	char			cur_sz[FMT_SCALED_STRSIZE];
@@ -1096,30 +1132,37 @@ xtp_page_dl_row(struct tab *t, char *html, struct download *dl)
 	case WEBKIT_DOWNLOAD_STATUS_CANCELLED:
 		status_html = g_strdup_printf("Cancelled");
 		cmd_html = g_strdup_printf(
-		    "<a href='%s%d/%d'>Remove</a> / <a href='%s%d/%d'>Unlink</a>",
+		    "<a href='%s%d/%d'>Restart</a> / <a href='%s%d/%d'>Remove</a> / <a href='%s%d/%d'>Unlink</a>",
+		    xtp_prefix, XT_XTP_DL_START, dl->id,
 		    xtp_prefix, XT_XTP_DL_REMOVE, dl->id, xtp_prefix,
 		    XT_XTP_DL_UNLINK, dl->id);
 		break;
 	case WEBKIT_DOWNLOAD_STATUS_ERROR:
 		status_html = g_strdup_printf("Error!");
 		cmd_html = g_strdup_printf(
-		    "<a href='%s%d/%d'>Remove</a> / <a href='%s%d/%d'>Unlink</a>",
+		    "<a href='%s%d/%d'>Restart</a> / <a href='%s%d/%d'>Remove</a> / <a href='%s%d/%d'>Unlink</a>",
+		    xtp_prefix, XT_XTP_DL_START, dl->id,
 		    xtp_prefix, XT_XTP_DL_REMOVE, dl->id, xtp_prefix,
 		    XT_XTP_DL_UNLINK, dl->id);
 		break;
 	case WEBKIT_DOWNLOAD_STATUS_CREATED:
-		cmd_html = g_strdup_printf("<a href='%s%d/%d'>Cancel</a>",
-		    xtp_prefix, XT_XTP_DL_CANCEL, dl->id);
-		status_html = g_strdup_printf("Starting");
+		cmd_html = g_strdup_printf("<a href='%s%d/%d'>Start</a> / <a href='%s%d/%d'>Cancel</a>",
+		    xtp_prefix, XT_XTP_DL_START, dl->id, xtp_prefix,
+		    XT_XTP_DL_CANCEL, dl->id);
+		status_html = g_strdup_printf("Created");
 		break;
 	default:
 		show_oops(t, "%s: unknown download status", __func__);
 	};
 
+	destination = webkit_download_get_destination_uri(dl->download);
+	/* we might not have a destination set yet */
+	if (!destination)
+		destination = webkit_download_get_suggested_filename(dl->download);
 	new_html = g_strdup_printf(
 	    "%s\n<tr><td>%s</td><td>%s</td>"
 	    "<td style='text-align:center'>%s</td></tr>\n",
-	    html, basename((char *)webkit_download_get_destination_uri(dl->download)),
+	    html, basename((char *)destination),
 	    status_html, cmd_html);
 	g_free(html);
 
@@ -1171,7 +1214,10 @@ xtp_page_cl(struct tab *t, struct karg *args)
 	pc = soup_cookie_jar_all_cookies(p_cookiejar);
 	pc_start = pc;
 
-	body = NULL;
+	body = g_strdup_printf("<div align=\"center\"><a href=\"%s%d/%s/%d\">"
+	    "[ Remove All Cookies From All Domains ]</a></div>\n",
+	    XT_XTP_STR, XT_XTP_CL, cl_session_key, XT_XTP_CL_REMOVE_ALL);
+
 	last_domain = strdup("");
 	for (; sc; sc = sc->next) {
 		c = sc->data;
@@ -1185,21 +1231,24 @@ xtp_page_cl(struct tab *t, struct karg *args)
 			if (body != NULL) {
 				tmp = body;
 				body = g_strdup_printf("%s</table>"
-				    "<h2>%s</h2>"
-				    "<a href='%s%d/%s/%d/%d'>remove all</a>%s\n",
+				    "<h2>%s</h2><div align=\"center\">"
+				    "<a href='%s%d/%s/%d/%d'>"
+				    "[ Remove All From This Domain ]"
+				    "</a></div>%s\n",
 				    body, c->domain,
-				    XT_XTP_STR, XT_XTP_CL,
-				    cl_session_key, XT_XTP_CL_REMOVE_DOMAIN, domain_id,
+				    XT_XTP_STR, XT_XTP_CL, cl_session_key,
+				    XT_XTP_CL_REMOVE_DOMAIN, domain_id,
 				    table_headers);
 				g_free(tmp);
 			} else {
 				/* first domain */
 				body = g_strdup_printf("<h2>%s</h2>"
-				    "<a href='%s%d/%s/%d/%d'>remove all</a>%s\n",
-				    c->domain,
-				    XT_XTP_STR, XT_XTP_CL,
-				    cl_session_key, XT_XTP_CL_REMOVE_DOMAIN, domain_id,
-				    table_headers);
+				    "<div align=\"center\">"
+				    "<a href='%s%d/%s/%d/%d'>"
+				    "[ Remove All From This Domain ]</a></div>%s\n",
+				    c->domain, XT_XTP_STR, XT_XTP_CL,
+				    cl_session_key, XT_XTP_CL_REMOVE_DOMAIN,
+				    domain_id, table_headers);
 			}
 		}
 
@@ -1464,3 +1513,181 @@ startpage_add(const char *fmt, ...)
 	TAILQ_INSERT_TAIL(&spl, s, entry);
 }
 
+gchar *
+show_g_object_settings(GObject *o, char *str, int recurse)
+{
+	char		*b, *body, *valstr;
+	guint		n_props = 0;
+	int		i;
+	GParamSpec	*pspec;
+	const gchar	*tname;
+	GValue		value;
+	int		typeno;
+	const gchar	*string;
+	gboolean	boolean;
+	gfloat		fp;
+	gdouble		fpd;
+	gint		number;
+	guint		unumber;
+	int64_t		number64;
+	uint64_t	unumber64;
+	GObject		*object;
+	GParamSpec	**proplist;
+	char		*tmpstr, *tmpsettings;
+
+	if (!G_IS_OBJECT(o)) {
+		fprintf(stderr, "%s is not a g_object\n", str);
+		return g_strdup("");
+	}
+	proplist = g_object_class_list_properties(
+	    G_OBJECT_GET_CLASS(o), &n_props);
+	body = g_strdup_printf("%s: %3d settings\n", str, n_props);
+	for (i=0; i < n_props; i++) {
+		pspec = proplist[i];
+		tname = G_OBJECT_TYPE_NAME(pspec);
+		bzero(&value, sizeof value);
+		valstr = NULL;
+
+		if (!(pspec->flags & G_PARAM_READABLE))
+			valstr = g_strdup_printf("not a readable property");
+		else {
+			g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
+			g_object_get_property(G_OBJECT(o), pspec->name,
+			    &value);
+		}
+
+		/* based on the type, recurse and display values */
+		if (valstr == NULL) {
+			typeno = G_TYPE_FUNDAMENTAL( G_VALUE_TYPE(&value) );
+			switch ( typeno ) {
+			case G_TYPE_ENUM:
+				number = g_value_get_enum(&value);
+				valstr = g_strdup_printf("%d", number);
+				break;
+			case G_TYPE_INT:
+				number = g_value_get_int(&value);
+				valstr = g_strdup_printf("%d", number);
+				break;
+			case G_TYPE_INT64:
+				number64 = (int64_t)g_value_get_int64(&value);
+				valstr = g_strdup_printf("%" PRIo64, number64);
+				break;
+			case G_TYPE_UINT:
+				unumber = g_value_get_uint(&value);
+				valstr = g_strdup_printf("%d", unumber);
+				break;
+			case G_TYPE_UINT64:
+				unumber64 =
+				    (uint64_t)g_value_get_uint64(&value);
+				valstr =
+				    g_strdup_printf("%" PRIu64, unumber64);
+				break;
+			case G_TYPE_FLAGS:
+				unumber = g_value_get_flags(&value);
+				valstr = g_strdup_printf("0x%x", unumber);
+				break;
+			case G_TYPE_BOOLEAN:
+				boolean = g_value_get_boolean(&value);
+				valstr = g_strdup_printf("%d", boolean);
+				break;
+			case G_TYPE_FLOAT:
+				fp = g_value_get_float(&value);
+				valstr = g_strdup_printf("%f", fp);
+				break;
+			case G_TYPE_DOUBLE:
+				fpd = g_value_get_double(&value);
+				valstr = g_strdup_printf("%f", fpd);
+				break;
+			case G_TYPE_STRING:
+				string = g_value_get_string(&value);
+				valstr = g_strdup_printf("%s", string);
+				break;
+			case G_TYPE_OBJECT:
+				object = g_value_get_object(&value);
+				if (object != NULL) {
+					if (recurse) {
+					tmpstr = g_strdup_printf("%s %s>", str,
+					    pspec->name);
+					tmpsettings = show_g_object_settings(
+					    object, tmpstr, recurse);
+					valstr = g_strdup_printf(
+					    "{\n%s%s     }\n",
+					    tmpsettings, str);
+					g_free(tmpstr);
+					g_free(tmpsettings);
+					} else {
+					valstr = g_strdup_printf("<...>");
+					}
+				} else {
+					valstr = g_strdup_printf("NULL");
+				}
+				break;
+			default:
+				valstr = g_strdup_printf("unhandled type '%s'",
+				    tname);
+			}
+		}
+
+		b = body;
+		body = g_strdup_printf(
+		    "%s%s: %3d: %s type=%s, flags=0x%x, val = %s\n",
+		    body, str, i, pspec->name, tname,
+		    pspec->flags, valstr);
+		g_free(b);
+		g_free(valstr);
+	}
+	g_free(proplist);
+	return (body);
+}
+
+int
+about_webkit(struct tab *t, struct karg *arg)
+{
+	char			*page, *body, *settingstr;
+
+	settingstr = show_g_object_settings(G_OBJECT(t->settings),
+	    "t->settings", 0);
+	body = g_strdup_printf("<pre>%s</pre>\n", settingstr);
+	g_free(settingstr);
+
+	page = get_html_page("About Webkit", body, "", 0);
+	g_free(body);
+
+	load_webkit_string(t, page, XT_URI_ABOUT_WEBKIT);
+	g_free(page);
+
+	return (0);
+}
+
+int
+allthethings(struct tab *t, struct karg *arg)
+{
+	char			*page, *body, *b, *settingstr;
+	extern GtkWidget	*main_window;
+
+	body = show_g_object_settings(G_OBJECT(t->wv), "t->wv", 1);
+	b = body;
+	settingstr = show_g_object_settings(G_OBJECT(t->inspector),
+	    "t->inspector", 1);
+	body = g_strdup_printf("%s%s", body, settingstr);
+	g_free(b);
+	g_free(settingstr);
+	b = body;
+	settingstr = show_g_object_settings(G_OBJECT(main_window),
+	    "main_window", 1);
+	body = g_strdup_printf("%s%s", body, settingstr);
+	g_free(b);
+	g_free(settingstr);
+	b = body;
+	body = g_strdup_printf("<pre>%scan paste clipboard = %d\n</pre>", body,
+	    webkit_web_view_can_paste_clipboard(t->wv));
+	g_free(b);
+
+	page = get_html_page("About All The Things _o/", body, "", 0);
+	g_free(body);
+
+	load_webkit_string(t, page, XT_URI_ABOUT_ALLTHETHINGS);
+	g_free(page);
+
+	return (0);
+}
