@@ -1,6 +1,4 @@
 #!/bin/sh
-# $xxxterm$
-#
 # This is an example script for playing flash videos. It requires the
 # get_flash_video package to be installed on the system. You can copy this
 # file to ~/.xxxterm/, and set
@@ -11,4 +9,4 @@
 # to avoid accumulating files:
 #   player = mplayer -loop 0 %s 2>/dev/null; rm -f %s
 
-cd /var/tmp && get_flash_videos -p "$1"
+cd /var/tmp && get_flash_videos --player="/usr/local/bin/mplayer -really-quiet %s 2> /dev/null" -q -p -y $1
