@@ -19,7 +19,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include "xxxterm.h"
+#include <xxxterm.h>
 
 /* globals */
 SoupURI			*proxy_uri = NULL;
@@ -63,11 +63,13 @@ int		cookie_policy = SOUP_COOKIE_JAR_ACCEPT_ALWAYS;
 char		*ssl_ca_file = NULL;
 char		*resource_dir = NULL;
 gboolean	ssl_strict_certs = FALSE;
+gboolean	enable_strict_transport = TRUE;
 int		append_next = 1; /* append tab after current tab */
 char		*home = NULL;
 char		*search_string = NULL;
 char		*http_proxy = NULL;
 char		download_dir[PATH_MAX];
+int		download_mode = XT_DM_START;
 char		runtime_settings[PATH_MAX]; /* override of settings */
 int		allow_volatile_cookies = 0;
 int		color_visited_uris = 1;
@@ -96,6 +98,8 @@ int		enable_autoscroll = 0;
 int		enable_favicon_entry = 1;
 int		enable_favicon_tabs = 0;
 char		*external_editor = NULL;
+int		referer_mode = XT_REFERER_ALWAYS;
+char		*referer_custom = NULL;
 
 char		*cmd_font_name = NULL;
 char		*oops_font_name = NULL;
@@ -107,7 +111,9 @@ char		*get_default_script(struct settings *);
 char		*get_runtime_dir(struct settings *);
 char		*get_tab_style(struct settings *);
 char		*get_edit_mode(struct settings *);
+char		*get_download_mode(struct settings *);
 char		*get_work_dir(struct settings *);
+char		*get_referer(struct settings *);
 
 int		add_cookie_wl(struct settings *, char *);
 int		add_js_wl(struct settings *, char *);
@@ -127,6 +133,10 @@ int		set_auto_load_images(char *value);
 int		set_enable_autoscroll(char *value);
 int		set_enable_favicon_entry(char *value);
 int		set_enable_favicon_tabs(char *value);
+int		set_download_mode(struct settings *, char *);
+int		set_download_mode_rt(char *);
+int		set_referer(struct settings *, char *);
+int		set_referer_rt(char *);
 int		set_external_editor(char *);
 
 void		walk_mime_type(struct settings *, void (*)(struct settings *,
@@ -261,12 +271,24 @@ struct special		s_edit_mode = {
 	NULL
 };
 
+struct special		s_download_mode = {
+	set_download_mode,
+	get_download_mode,
+	NULL
+};
+
 struct special		s_ua = {
 	add_ua,
 	NULL,
 	walk_ua
 };
 
+struct special		s_referer = {
+	set_referer,
+	get_referer,
+	NULL
+};
+
 struct settings		rs[] = {
 	{ "allow_volatile_cookies",	XT_S_INT, 0,		&allow_volatile_cookies, NULL, NULL },
 	{ "append_next",		XT_S_INT, 0,		&append_next, NULL, NULL },
@@ -280,6 +302,7 @@ struct settings		rs[] = {
 	{ "default_zoom_level",		XT_S_FLOAT, 0,		NULL, NULL, NULL, &default_zoom_level },
 	{ "default_script",		XT_S_STR, 0, NULL, NULL,&s_default_script },
 	{ "download_dir",		XT_S_STR, 0, NULL, NULL,&s_download_dir },
+	{ "download_mode",		XT_S_STR, 0, NULL, NULL,&s_download_mode, NULL, set_download_mode_rt },
 	{ "edit_mode",			XT_S_STR, 0, NULL, NULL,&s_edit_mode },
 	{ "enable_cookie_whitelist",	XT_S_INT, 0,		&enable_cookie_whitelist, NULL, NULL },
 	{ "enable_js_whitelist",	XT_S_INT, 0,		&enable_js_whitelist, NULL, NULL },
@@ -315,18 +338,21 @@ struct settings		rs[] = {
 	{ "spell_check_languages",	XT_S_STR, 0, NULL, &spell_check_languages, NULL },
 	{ "ssl_ca_file",		XT_S_STR, 0, NULL,	&ssl_ca_file, NULL },
 	{ "ssl_strict_certs",		XT_S_INT, 0,		&ssl_strict_certs, NULL, NULL },
+	{ "enable_strict_transport",	XT_S_INT, 0,		&enable_strict_transport, NULL, NULL },
 	{ "statusbar_elems",		XT_S_STR, 0, NULL,	&statusbar_elems, NULL },
 	{ "tab_style",			XT_S_STR, 0, NULL, NULL,&s_tab_style },
+	{ "userstyle_global",		XT_S_INT, 0,		&userstyle_global, NULL, NULL },
 	{ "url_regex",			XT_S_STR, 0, NULL,	&url_regex, NULL },
 	{ "window_height",		XT_S_INT, 0,		&window_height, NULL, NULL },
 	{ "window_width",		XT_S_INT, 0,		&window_width, NULL, NULL },
 	{ "window_maximize",		XT_S_INT, 0,		&window_maximize, NULL, NULL },
 	{ "work_dir",			XT_S_STR, 0, NULL, NULL,&s_work_dir },
 	{ "xterm_workaround",		XT_S_INT, 0,		&xterm_workaround, NULL, NULL },
-	{ "auto_load_images",		XT_S_INT, 0, 		&auto_load_images, NULL, NULL, NULL, set_auto_load_images },
-	{ "enable_autoscroll",		XT_S_INT, 0, 		&enable_autoscroll, NULL, NULL, NULL, set_enable_autoscroll },
-	{ "enable_favicon_entry",	XT_S_INT, 0, 		&enable_favicon_entry, NULL, NULL, NULL, set_enable_favicon_entry },
-	{ "enable_favicon_tabs",	XT_S_INT, 0, 		&enable_favicon_tabs, NULL, NULL, NULL, set_enable_favicon_tabs },
+	{ "auto_load_images",		XT_S_INT, 0,		&auto_load_images, NULL, NULL, NULL, set_auto_load_images },
+	{ "enable_autoscroll",		XT_S_INT, 0,		&enable_autoscroll, NULL, NULL, NULL, set_enable_autoscroll },
+	{ "enable_favicon_entry",	XT_S_INT, 0,		&enable_favicon_entry, NULL, NULL, NULL, set_enable_favicon_entry },
+	{ "enable_favicon_tabs",	XT_S_INT, 0,		&enable_favicon_tabs, NULL, NULL, NULL, set_enable_favicon_tabs },
+	{ "referer",			XT_S_STR, 0, NULL, NULL,&s_referer, NULL, set_referer_rt },
 
 	/* font settings */
 	{ "cmd_font",			XT_S_STR, 0, NULL, &cmd_font_name, NULL },
@@ -417,6 +443,7 @@ set_browser_mode(struct settings *s, char *val)
 		enable_scripts = 0;
 		enable_js_whitelist = 1;
 		enable_localstorage = 0;
+		referer_mode = XT_REFERER_SAME_DOMAIN;
 	} else if (!strcmp(val, "normal")) {
 		browser_mode = XT_BM_NORMAL;
 		allow_volatile_cookies = 0;
@@ -431,6 +458,7 @@ set_browser_mode(struct settings *s, char *val)
 		enable_scripts = 1;
 		enable_js_whitelist = 0;
 		enable_localstorage = 1;
+		referer_mode = XT_REFERER_ALWAYS;
 	} else if (!strcmp(val, "kiosk")) {
 		browser_mode = XT_BM_KIOSK;
 		allow_volatile_cookies = 0;
@@ -445,6 +473,7 @@ set_browser_mode(struct settings *s, char *val)
 		enable_scripts = 1;
 		enable_js_whitelist = 0;
 		enable_localstorage = 1;
+		referer_mode = XT_REFERER_ALWAYS;
 		show_tabs = 0;
 		tabless = 1;
 	} else
@@ -550,7 +579,7 @@ int
 set_default_script(struct settings *s, char *val)
 {
 	if (val[0] == '~')
-		snprintf(default_script, sizeof default_script, "%s/%s",
+		snprintf(default_script, sizeof default_script, "%s" PS "%s",
 		    pwd->pw_dir, &val[1]);
 	else
 		strlcpy(default_script, val, sizeof default_script);
@@ -570,7 +599,7 @@ int
 set_download_dir(struct settings *s, char *val)
 {
 	if (val[0] == '~')
-		snprintf(download_dir, sizeof download_dir, "%s/%s",
+		snprintf(download_dir, sizeof download_dir, "%s" PS "%s",
 		    pwd->pw_dir, &val[1]);
 	else
 		strlcpy(download_dir, val, sizeof download_dir);
@@ -736,6 +765,7 @@ struct key_binding	keys[] = {
 	{ "run_script",		MOD1,	1,	GDK_r		},
 	{ "proxy toggle",	0,	1,	GDK_F2		},
 	{ "editelement",	CTRL,	1,	GDK_i		},
+	{ "passthrough",	CTRL,	1,	GDK_z		},
 
 	/* search */
 	{ "searchnext",		0,	0,	GDK_n		},
@@ -1064,6 +1094,51 @@ set_enable_favicon_tabs(char *value)
 	return (0);
 }
 
+char *
+get_referer(struct settings *s)
+{
+	if (referer_mode == XT_REFERER_ALWAYS)
+		return (g_strdup("always"));
+	if (referer_mode == XT_REFERER_NEVER)
+		return (g_strdup("never"));
+	if (referer_mode == XT_REFERER_SAME_DOMAIN)
+		return (g_strdup("same-domain"));
+	if (referer_mode == XT_REFERER_CUSTOM)
+		return (g_strdup(referer_custom));
+	return (NULL);
+}
+
+int
+set_referer(struct settings *s, char *value)
+{
+	if (referer_custom)
+		g_free(referer_custom);
+
+	if (!strcmp(value, "always"))
+		referer_mode = XT_REFERER_ALWAYS;
+	else if (!strcmp(value, "never"))
+		referer_mode = XT_REFERER_NEVER;
+	else if (!strcmp(value, "same-domain"))
+		referer_mode = XT_REFERER_SAME_DOMAIN;
+	else if (!valid_url_type(value)) {
+		referer_mode = XT_REFERER_CUSTOM;
+		referer_custom = g_strdup(value);
+	} else {
+		/* we've already free'd the custom referer */
+		if (referer_mode == XT_REFERER_CUSTOM)
+			referer_mode = XT_REFERER_NEVER;
+		return (1);
+	}
+
+	return (0);
+}
+
+int
+set_referer_rt(char *value)
+{
+	return set_referer(NULL, value);
+}
+
 int
 set_external_editor(char *editor)
 {
@@ -1144,6 +1219,44 @@ set_edit_mode(struct settings *s, char *val)
 	return (0);
 }
 
+char *
+get_download_mode(struct settings *s)
+{
+	switch (download_mode) {
+	case XT_DM_START:
+		return (g_strdup("start"));
+		break;
+	case XT_DM_ASK:
+		return (g_strdup("ask"));
+		break;
+	case XT_DM_ADD:
+		return (g_strdup("add"));
+		break;
+	}
+	return (g_strdup("unknown"));
+}
+
+int
+set_download_mode(struct settings *s, char *val)
+{
+	if (!strcmp(val, "start"))
+		download_mode = XT_DM_START;
+	else if (!strcmp(val, "ask"))
+		download_mode = XT_DM_ASK;
+	else if (!strcmp(val, "add"))
+		download_mode = XT_DM_ADD;
+	else
+		return (1);
+
+	return (0);
+}
+
+int
+set_download_mode_rt(char *val)
+{
+	return set_download_mode(NULL, val);
+}
+
 char *
 get_work_dir(struct settings *s)
 {
@@ -1156,7 +1269,7 @@ int
 set_work_dir(struct settings *s, char *val)
 {
 	if (val[0] == '~')
-		snprintf(work_dir, sizeof work_dir, "%s/%s",
+		snprintf(work_dir, sizeof work_dir, "%s" PS "%s",
 		    pwd->pw_dir, &val[1]);
 	else
 		strlcpy(work_dir, val, sizeof work_dir);
@@ -1274,7 +1387,7 @@ config_parse(char *filename, int runtime)
 		return;
 
 	if (runtime && runtime_settings[0] != '\0') {
-		snprintf(file, sizeof file, "%s/%s",
+		snprintf(file, sizeof file, "%s" PS "%s",
 		    work_dir, runtime_settings);
 		if (stat(file, &sb)) {
 			warnx("runtime file doesn't exist, creating it");
