@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2011 Conformal Systems LLC <info@conformal.com>
  * Copyright (c) 2011 Marco Peereboom <marco@peereboom.us>
+ * Copyright (c) 2010, 2011, 2012 Edd Barrett <vext01@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -29,9 +30,9 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <fcntl.h>
+#include <inttypes.h>
 
 #include <sys/types.h>
-#include <sys/wait.h>
 #if defined(__linux__)
 #include "linux/util.h"
 #include "linux/tree.h"
@@ -44,12 +45,16 @@ u_int32_t	arc4random_uniform(u_int32_t);
 #include <libutil.h>
 #include "freebsd/util.h"
 #include <sys/tree.h>
-#else /* OpenBSD */
+#elif defined(__OpenBSD__)
 #include <util.h>
 #include <sys/tree.h>
+#else
+void		arc4random_buf(void *, size_t);
+u_int32_t	arc4random_uniform(u_int32_t);
+#include "util.h"
+#include "tree.h"
 #endif
 #include <sys/queue.h>
-#include <sys/resource.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -77,7 +82,16 @@ u_int32_t	arc4random_uniform(u_int32_t);
 #include <pthread.h>
 #endif
 
-#include "version.h"
+/* set if you don't want to use signals */
+#ifndef XT_SIGNALS_DISABLE
+#include <sys/wait.h>
+#endif
+
+/* set if you don't want to use resource limits */
+#ifndef XT_RESOURCE_LIMITS_DISABLE
+#include <sys/resource.h>
+#endif
+
 #include "javascript.h"
 /*
 javascript.h borrowed from vimprobable2 under the following license:
@@ -131,7 +145,7 @@ THE SOFTWARE.
 #define XT_D_INSPECTOR		0x8000
 #define XT_D_VISITED		0x10000
 #define XT_D_HISTORY		0x20000
-extern u_int32_t	swm_debug;
+extern uint32_t	swm_debug;
 #else
 #define DPRINTF(x...)
 #define DNPRINTF(n,x...)
@@ -147,6 +161,11 @@ extern u_int32_t	swm_debug;
 
 #define XT_NOMARKS		(('z' - 'a' + 1) * 2 + 10)
 
+/* Path Separator */
+#ifndef	PS
+#define PS			"/"
+#endif
+
 struct tab {
 	TAILQ_ENTRY(tab)	entry;
 	GtkWidget		*vbox;
@@ -202,6 +221,9 @@ struct tab {
 #define XT_MODE_COMMAND		(0)
 #define XT_MODE_INSERT		(1)
 #define XT_MODE_HINT		(2)
+#define XT_MODE_PASSTHROUGH	(3)
+	int			(*mode_cb)(struct tab *, GdkEventKey *, gpointer);
+	gpointer		mode_cb_data;
 	int			focus_wv;
 	int			ctrl_click;
 	gchar			*status;
@@ -264,11 +286,25 @@ struct history {
 RB_HEAD(history_list, history);
 RB_PROTOTYPE(history_list, history, entry, history_rb_cmp);
 
+#define XT_STS_FLAGS_INCLUDE_SUBDOMAINS		(1)
+#define XT_STS_FLAGS_EXPAND			(2)
+#define XT_STS_FILE				("strict-transport")
+
+struct strict_transport {
+	RB_ENTRY(strict_transport)	entry;
+	gchar				*host;
+	time_t				timeout;
+	int				flags;
+};
+RB_HEAD(strict_transport_tree, strict_transport);
+RB_PROTOTYPE(strict_transport_tree, strict_transport, entry, strict_transport_rb_cmp);
+
 /* utility */
 #define XT_NAME			("XXXTerm")
 #define XT_CB_HANDLED		(TRUE)
 #define XT_CB_PASSTHROUGH	(FALSE)
 #define XT_FAVS_FILE		("favorites")
+#define XT_SOD_FILE		("startofday")
 
 int			run_script(struct tab *, char *);
 void			xt_icon_from_file(struct tab *, char *);
@@ -279,6 +315,7 @@ const gchar		*get_uri(struct tab *);
 const gchar		*get_title(struct tab *, bool);
 void			load_uri(struct tab *t, gchar *uri);
 gboolean		match_uri(const gchar *uri, const gchar *key);
+int			valid_url_type(char *);
 
 void			load_webkit_string(struct tab *, const char *, gchar *);
 void			button_set_stockid(GtkWidget *, char *);
@@ -286,6 +323,7 @@ void			button_set_stockid(GtkWidget *, char *);
 /* cookies */
 int			remove_cookie(int);
 int			remove_cookie_domain(int);
+int			remove_cookie_all(void);
 void			print_cookie(char *msg, SoupCookie *);
 void			setup_cookies(void);
 
@@ -301,6 +339,7 @@ void			completion_add(struct tab *);
 void			completion_add_uri(const gchar *uri);
 
 /* external editor */
+#define XT_EE_BUFSZ	(64 * 1024)
 int			edit_src(struct tab *t, struct karg *args);
 int			edit_element(struct tab *t, struct karg *a);
 
@@ -318,11 +357,16 @@ WebKitWebView*		inspector_inspect_web_view_cb(WebKitWebInspector *,
 void			setup_inspector(struct tab *);
 int			inspector_cmd(struct tab *, struct karg *);
 
+/* tld public suffix */
+void			tld_tree_init();
+char			*tld_get_suffix(const char *);
+
 /* about */
 #define XT_XTP_STR		"xxxt://"
 #define XT_URI_ABOUT		("about:")
 #define XT_URI_ABOUT_LEN	(strlen(XT_URI_ABOUT))
 #define XT_URI_ABOUT_ABOUT	("about")
+#define XT_URI_ABOUT_ALLTHETHINGS ("allthethings")
 #define XT_URI_ABOUT_BLANK	("blank")
 #define XT_URI_ABOUT_CERTS	("certs")
 #define XT_URI_ABOUT_COOKIEWL	("cookiewl")
@@ -337,6 +381,7 @@ int			inspector_cmd(struct tab *, struct karg *);
 #define XT_URI_ABOUT_STATS	("stats")
 #define XT_URI_ABOUT_MARCO	("marco")
 #define XT_URI_ABOUT_STARTPAGE	("startpage")
+#define XT_URI_ABOUT_WEBKIT	("webkit")
 
 struct about_type {
 	char		*name;
@@ -444,6 +489,9 @@ void		input_focus_blur(struct tab *, void *);
 void		*input_check_mode(struct tab *);
 int		command_mode(struct tab *, struct karg *);
 
+/* OS specific */
+int		fork_exec(struct tab *, char *, const gchar *, char *, int);
+
 /* settings */
 #define XT_BM_NORMAL		(0)
 #define XT_BM_WHITELIST		(1)
@@ -458,6 +506,15 @@ int		command_mode(struct tab *, struct karg *);
 #define XT_EM_HYBRID		(0)
 #define XT_EM_VI		(1)
 
+#define XT_DM_START		(0)
+#define XT_DM_ASK		(1)
+#define XT_DM_ADD		(2)
+
+#define XT_REFERER_ALWAYS	(0)
+#define XT_REFERER_NEVER	(1)
+#define XT_REFERER_SAME_DOMAIN	(2)
+#define XT_REFERER_CUSTOM	(3)
+
 #define CTRL			GDK_CONTROL_MASK
 #define MOD1			GDK_MOD1_MASK
 #define SHFT			GDK_SHIFT_MASK
@@ -508,6 +565,10 @@ void		init_keybindings(void);
 void		config_parse(char *, int);
 char		*get_setting_name(int);
 
+#define		XT_DL_START	(0)
+#define		XT_DL_RESTART	(1)
+int		download_start(struct tab *, struct download *, int flag);
+
 extern int	tabless;
 extern int	enable_socket;
 extern int	single_instance;
@@ -540,12 +601,14 @@ extern int	cookie_policy;
 extern char	*ssl_ca_file;
 extern char	*resource_dir;
 extern gboolean	ssl_strict_certs;
+extern gboolean	enable_strict_transport;
 extern int	append_next;
 extern char	*home;
 extern char	*search_string;
 extern char	*http_proxy;
 extern char	*external_editor;
 extern char	download_dir[PATH_MAX];
+extern int	download_mode;
 extern char	runtime_settings[PATH_MAX];
 extern int	allow_volatile_cookies;
 extern int	color_visited_uris;
@@ -577,6 +640,8 @@ extern int	auto_load_images;
 extern int	enable_autoscroll;
 extern int	enable_favicon_entry;
 extern int	enable_favicon_tabs;
+extern int	referer_mode;
+extern char	*referer_custom;
 
 /* globals */
 extern char		*version;
@@ -585,12 +650,12 @@ extern char		rc_fname[PATH_MAX];
 extern char		work_dir[PATH_MAX];
 extern char		temp_dir[PATH_MAX];
 extern struct passwd	*pwd;
-long long unsigned int	blocked_cookies;
 extern SoupCookieJar	*s_cookiejar;
 extern SoupCookieJar	*p_cookiejar;
 extern SoupSession	*session;
 extern GtkNotebook	*notebook;
 extern GtkListStore	*completion_model;
+extern uint64_t		blocked_cookies;
 
 extern void	(*_soup_cookie_jar_add_cookie)(SoupCookieJar *, SoupCookie *);
 
@@ -602,6 +667,7 @@ extern struct about_type	about_list[];
 extern struct domain_list	c_wl;
 extern struct domain_list	js_wl;
 extern struct domain_list	pl_wl;
+extern struct strict_transport_tree	st_tree;
 extern struct alias_list	aliases;
 extern struct mime_type_list	mtl;
 extern struct keybinding_list	kbl;
